{% extends "wagtailadmin/base.html" %}
{% load i18n %}

{% block titletag %}{% trans "Change password" %}{% endblock %}
{% block content %}
    {% trans "Change password" as change_str %}
    {% include "wagtailadmin/shared/header.html" with title=change_str %}

    <div class="nice-padding">
        {% if can_change_password %}
            <form action="{% url 'wagtailadmin_account_change_password' %}" method="POST" novalidate>
                {% csrf_token %}
                <ul class="fields">
                    {% for field in form %}
                        {% include "wagtailadmin/shared/field_as_li.html" with field=field %}
                    {% endfor %}
                </ul>
                <input type="submit" value="{% trans 'Change Password' %}" class="button" />
            </form>
        {% else %}
            <p>{% trans "Your password can't be changed here. Please contact a site administrator." %}</p>
        {% endif %}
    </div>
{% endblock %}
