{% load i18n wagtailadmin_tags %}

<li class="icon icon-doc-full-inverse">
    <a href="{% url 'wagtaildocs:index' %}">
    {% blocktrans count counter=total_docs with total_docs|intcomma as total %}
        <span>{{ total }}</span> Document
    {% plural %}
        <span>{{ total }}</span> Documents
    {% endblocktrans %}
    </a>
</li>
