{% comment %}
The title field for a page in the page listing, when in 'choose' mode.

Expects a variable 'page', the page instance.
{% endcomment %}

<h2>
    {% if page.can_choose %}
        <a class="choose-page" href="#{{ page.id }}" data-id="{{ page.id }}" data-title="{{ page.title }}" data-url="{{ page.url }}" data-parent-id="{{ page.get_parent.id }}" data-edit-url="{% url 'wagtailadmin_pages:edit' page.id %}">{{ page.title }}</a>
    {% else %}
        {{ page.title }}
    {% endif %}

    {% include "wagtailadmin/pages/listing/_privacy_indicator.html" with page=page %}
    {% include "wagtailadmin/pages/listing/_locked_indicator.html" with page=page %}
</h2>
