from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailMedusaAppConfig(AppConfig):
    name = 'wagtail.contrib.wagtailmedusa'
    label = 'wagtailmedusa'
    verbose_name = "Wagtail medusa"
