{% load wagtailadmin_tags staticfiles %}

{% comment %}
    CSS declarations to be included on the 'create page' and 'edit page' views
{% endcomment %}

<link rel="stylesheet" href="{% static 'wagtailadmin/css/layouts/page-editor.css' %}" type="text/css" />
<link rel="stylesheet" href="{% static 'wagtailadmin/css/panels/rich-text.css' %}" type="text/css" />

{% hook_output 'insert_editor_css' %}

{{ edit_handler.form.media.css }}
