from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailRedirectsAppConfig(AppConfig):
    name = 'wagtail.wagtailredirects'
    label = 'wagtailredirects'
    verbose_name = "Wagtail redirects"
