{% extends "wagtailadmin/base.html" %}
{% load i18n %}

{% block titletag %}{% trans "Account" %}{% endblock %}
{% block content %}
    {% trans "Account" as account_str %}
    {% include "wagtailadmin/shared/header.html" with title=account_str %}

    <div class="nice-padding">
        <ul class="listing">
            <li class="row row-flush">
                <div class="col6">
                    <a href="https://gravatar.com/emails/" class="button button-primary">{% trans "Set gravatar" %}</a>
                </div>

                <small class="col6">
                    {% trans "Your avatar image is provided by Gravatar and is connected to your email address. With a Gravatar account you can set an avatar for any number of other email addresses you use." %}
                </small>
                
            </li>
            {% if show_change_password %}
                <li class="row row-flush">
                    <div class="col6">
                        <a href="{% url 'wagtailadmin_account_change_password' %}" class="button button-primary">{% trans "Change password" %}</a>
                    </div>

                    <small class="col6">
                        {% trans "Change the password you use to log in." %}
                    </small>
                </li>
            {% endif %}
            {% if show_notification_preferences %}
                <li class="row row-flush">
                    <div class="col6">
                        <a href="{% url 'wagtailadmin_account_notification_preferences' %}" class="button button-primary">{% trans "Notification preferences" %}</a>
                    </div>

                    <small class="col6">
                        {% trans "Choose which email notifications to receive." %}
                    </small>
                </li>
            {% endif %}
        </ul>
    </div>
{% endblock %}
