from __future__ import absolute_import, unicode_literals

from wagtail.wagtailadmin.forms import WagtailAdminModelForm


class FancySnippetForm(WagtailAdminModelForm):
    """
    A custom form class for FancySnippets in the admin
    """
