{% comment %}
The title field for a page in the page listing, when in 'move' mode.

Expects a variable 'page', the page instance.
{% endcomment %}

<h2>
    {% if page.can_choose %}
        <a href="{% url 'wagtailadmin_pages:move_confirm' page_to_move.id page.id %}">{{ page.title }}</a>
    {% else %}
        {{ page.title }}
    {% endif %}

    {% include "wagtailadmin/pages/listing/_privacy_indicator.html" with page=page %}
    {% include "wagtailadmin/pages/listing/_locked_indicator.html" with page=page %}
</h2>
