<!DOCTYPE HTML>
<html>
    <head>
        <title>Password required</title>
    </head>
    <body>
        <h1>Password required</h1>
        <p>You need a password to access this page.</p>
        <form action="{{ action_url }}" method="POST">
            {% csrf_token %}
            {{ form.as_p }}
            <input type="submit" value="Continue" class="button" />
        </form>
    </body>
</html>
