{% load i18n wagtailusers_tags %}
{% load gravatar %}
<table class="listing">
    <thead>
        <tr>
            <th class="name">
                {% trans "Name" %}
                {% if ordering == "name" %}
                    <a href="{% url 'wagtailusers_users:index' %}" class="icon icon-arrow-down-after teal"></a>
                {% else %}
                    <a href="{% url 'wagtailusers_users:index' %}?ordering=name" class="icon icon-arrow-down-after"></a>
                {% endif %}
            </th>
            <th class="username">
                {% trans "Username" %}
                {% if ordering == "username" %}
                    <a href="{% url 'wagtailusers_users:index' %}" class="icon icon-arrow-down-after teal"></a>
                {% else %}
                    <a href="{% url 'wagtailusers_users:index' %}?ordering=username" class="icon icon-arrow-down-after"></a>
                {% endif %}
            </th>
            <th class="level">{% trans "Level" %}</th>
            <th class="status">{% trans "Status" %}</th>
        </tr>
    </thead>
    <tbody>
        {% for user in users %}
            <tr>
                <td class="title" valign="top">
                    <h2>
                        <span class="avatar small icon icon-user"><img src="{% gravatar_url user.email 25 %}" /></span>
                        <a href="{% url 'wagtailusers_users:edit' user.pk %}">{{ user.get_full_name|default:user.get_username }}</a>
                    </h2>
                    <ul class="actions">
                        {% user_listing_buttons user %}
                    </ul>
                </td>
                <td class="username" valign="top">{{ user.get_username }}</td>
                <td class="level" valign="top">{% if user.is_superuser %}{% trans "Admin" %}{% endif %}</td>
                <td class="status" valign="top"><div class="status-tag {% if user.is_active %}primary{% endif %}">{% if user.is_active %}{% trans "Active" %}{% else %}{% trans "Inactive" %}{% endif %}</div></td>
            </tr>
        {% endfor %}
    </tbody>
</table>
