from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailRoutablePageAppConfig(AppConfig):
    name = 'wagtail.contrib.wagtailroutablepage'
    label = 'wagtailroutablepage'
    verbose_name = "Wagtail routablepage"
