from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailCoreAppConfig(AppConfig):
    name = 'wagtail.wagtailcore'
    label = 'wagtailcore'
    verbose_name = "Wagtail core"
