{% extends "tests/base.html" %}

{% block content %}
    <p>This event is invitation only. Please enter your password to see the details.</p>
    <form action="{{ action_url }}" method="POST">
        {% csrf_token %}
        {{ form.as_p }}
        <input type="submit" value="Continue" class="button" />
    </form>
{% endblock %}
