from __future__ import absolute_import, unicode_literals

import warnings

from wagtail.wagtailcore.blocks import *  # noqa

warnings.warn("wagtail.wagtailadmin.blocks has moved to wagtail.wagtailcore.blocks", UserWarning, stacklevel=2)
