<!DOCTYPE HTML>
{% load staticfiles i18n %}
<html>
    <head>
        <title>{% trans "Preview" %}</title>
        <link rel="stylesheet" href="{% static 'wagtailadmin/css/layouts/preview.css' %}" type="text/css" />
    </head>
    <body class="ready">
        <div id="loading-spinner-wrapper" class="loading-spinner-wrapper">
            <div id="loading-spinner" class="loading-spinner"></div>
        </div>
        <iframe id="preview-frame" class="preview-frame" src="{% url 'wagtailadmin_pages:preview_loading' %}"></iframe>
    </body>
</html>
