{% extends "wagtailadmin/widgets/chooser.html" %}

{% block chosen_state_view %}
    <span class="title">{{ page.title }}</span>
{% endblock %}

{% block edit_chosen_item_url %}{% if page %}{% url 'wagtailadmin_pages:edit' page.id %}{% endif %}{% endblock %}
