.listing {
    .field label {
        display: none;
    }

    input,
    select,

    textarea {
        font-size: 1em;
    }

    select + span:after {
        font-size: 2.5em !important;
    }
}
