from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailDocsAppConfig(AppConfig):
    name = 'wagtail.wagtaildocs'
    label = 'wagtaildocs'
    verbose_name = "Wagtail documents"
